@@ -2,27 +2,33 @@
 #
 # SPDX-License-Identifier: EUPL-1.2
 
-import datetime
-from minty.entity import Entity, EntityCollection
+import orjson
+from minty.entity import Entity, EntityCollection, ValueObject
 from pyramid.renderers import JSON
 from typing import Any, Dict
-from uuid import UUID
 
 
 class jsonapi(JSON):
     jsonapi_adapters = {
         Entity: "_entity_adapter",
         EntityCollection: "_entity_collection_adapter",
-        UUID: "_uuid_adapter",
-        datetime.datetime: "_datetime_adapter",
+        ValueObject: "_value_object_adapter",
+        set: "_set_adapter",
     }
+    view: Any
 
     def __init__(self, *args, **kwargs):
         """Constructs the jsonapi class based on Pyramid JSON implementation.
 
-        This will add some default adapters to the system, like UUID, entity
-        parsing and datetime handling
+        This will add some default adapters to the system, to handle entities,
+        value objects, etc.
+
+        It will also replace the default JSON serializer with the `orjson`
+        one, which is faster and can handle more types (datetime, UUID, among
+        others)
         """
+
+        kwargs["serializer"] = orjson.dumps
         super().__init__(*args, **kwargs)
 
         for obj, adapter in self.jsonapi_adapters.items():
@@ -46,6 +52,7 @@ class jsonapi(JSON):
                 if ct == response.default_content_type:
                     response.content_type = "application/json"
             default = self._make_default(request)
+
             return self.serializer(value, default=default, **self.kw)
 
         return _render
@@ -173,24 +180,28 @@ class jsonapi(JSON):
         return {
             "type": obj.entity_type,
             "id": obj.entity_id,
-            "meta": {"summary": obj.entity_meta_summary},
             "attributes": object_attributes,
+            "meta": self._build_meta(obj),
             **relationships_block,
             **self.__links_for_entity(obj),
         }
 
-    def _uuid_adapter(self, obj, request):
-        """Transforms a UUID in a JSONApi styled format
+    def _build_meta(self, obj: Entity) -> dict:
+        meta = {}
+
+        for field_name in obj.entity_meta__fields:
+            short_field_name = field_name.replace("entity_meta_", "")
+            meta[short_field_name] = getattr(obj, field_name)
+
+        return meta
+
+    def _set_adapter(self, obj, request):
+        return list(sorted(obj))
+
+    def _value_object_adapter(self, obj, request):
+        """Transforms a ValueObject in a JSONApi styled format
 
         See renderers documentation of Pyramid for more information about
         using adapters
         """
-        return str(obj)
-
-    def _datetime_adapter(self, obj, request):
-        """Transforms a datetime object in a JSONApi styled format
-
-        See renderers documentation of Pyramid for more information about
-        using adapters
-        """
-        return obj.isoformat()
+        return obj.dict()
