@@ -2,15 +2,14 @@
 #
 # SPDX-License-Identifier: EUPL-1.2
 
-from collections import namedtuple
+from minty.cqrs import UserInfo
+from uuid import uuid4
 
 
 def mock_protected_route(view):
     def view_wrapper(request):
-        UserInfo = namedtuple("UserInfo", "user_uuid permissions")
-
         user_info = UserInfo(
-            user_uuid="subject_uuid", permissions="permissions"
+            user_uuid=uuid4(), permissions={"permission": True}
         )
 
         return view(request=request, user_info=user_info)
