@@ -4,18 +4,67 @@
 
 import logging
 import re
+from minty.cqrs import UserInfo
+from minty.validation import assert_never
 from minty_pyramid.session_manager import (
     check_user_permission,
     get_logged_in_user,
 )
 from pyramid.httpexceptions import HTTPBadRequest, HTTPForbidden
+from typing import Dict, Literal, Optional, Set, TypedDict
+from uuid import UUID
 
 log = logging.getLogger(__name__)
 
+# Default page size if none is specified.
+# Note that this is a str() because it's a default for a query parameter.
+DEFAULT_PAGE_SIZE = str(50)
+
+
+SupportedHTTPMethods = Literal["GET", "POST"]
+CQType = Literal["command", "query"]
+
+
+# Until PEP-655 lands, we need this workaround for optional keys in a TypedDict
+class FromDefinition(TypedDict, total=False):
+    request_params: dict
+    json: dict
+
+
+class PagingDefinitionBase(TypedDict):
+    max_results: int
+
+
+class PagingDefinition(PagingDefinitionBase, total=False):
+    page: Literal["page"]
+    page_size: Literal["page_size"]
+
+
+ViewMappingBase = TypedDict(
+    "ViewMappingBase",
+    {
+        "cq": CQType,
+        "auth_permissions": Set[str],
+        "domain": str,
+        "run": str,
+        "from": FromDefinition,
+    },
+)
+
+
+class ViewMapping(ViewMappingBase, total=False):
+    entity_type: str
+    command_primary: str
+    paging: PagingDefinition
+
+
+ViewMapper = Dict[SupportedHTTPMethods, Dict[str, ViewMapping]]
+
 
 # This will be moved to minty-pyramid in a next iteration
 class JSONAPIEntityView:
-    """View Base class for Pyramid Views which return Entity objects
+    """
+    View Base class for Pyramid Views which return Entity objects
 
     Use this as a base class for your own view. E.g.
 
@@ -72,7 +121,7 @@ class JSONAPIEntityView:
     Other parameters:
         cq (Enum("query", "command")):  Defines whether to call on the Query or
                                         Command class
-        auth_permissions (List[str]): The list of permissions to check
+        auth_permissions (Set[str]): The list of permissions to check
         domain (str): The domain to run this command or query against
         run (str): the command or query to run
         from (dict): A dict of parameters, see below
@@ -99,7 +148,7 @@ class JSONAPIEntityView:
 
     """
 
-    view_mapper: dict
+    view_mapper: ViewMapper
 
     def __init__(self, context, request):
         """Initializes this object from Pyramid"""
@@ -118,7 +167,7 @@ class JSONAPIEntityView:
 
         return user_info
 
-    def _get_cqrs_instance(self, calldata, user_info):
+    def _get_cqrs_instance(self, calldata: ViewMapping, user_info: UserInfo):
         """Gets the command or query instance for the called api view"""
         cq = calldata["cq"]
 
@@ -140,21 +189,70 @@ class JSONAPIEntityView:
         run = getattr(cqinstance, calldata["run"])
         params = self._collect_params(calldata)
 
-        response = None
-        if "wrapper" in calldata:
-            wrapper = getattr(self, calldata["wrapper"])
-            response = wrapper(run, params)
-        else:
-            response = run(**params)
+        response = run(**params)
 
         return self.transform_response_from_command_or_query(
             response, calldata, params
         )
 
-    def transform_response_from_command_or_query(
-        self, response, calldata, params
+    def transform_response_from_entity(
+        self, entity_type: str, entity_id: UUID
     ):
-        """Transforms a response from a command to a proper JSONApi structure
+        """Transform response from entity"""
+
+        link = self.create_link_from_entity(
+            entity_type=entity_type, entity_id=entity_id,
+        )
+
+        if link:
+            links_block = {"links": {"self": link}}
+        else:
+            links_block = {}
+
+        return {
+            **links_block,
+            "type": entity_type,
+            "id": entity_id,
+        }
+
+    def transform_response_from_command(
+        self, calldata: ViewMapping, params
+    ) -> dict:
+        """Transform response from command"""
+
+        if "command_primary" not in calldata or "entity_type" not in calldata:
+            return {"data": None}
+
+        if params.get(calldata["command_primary"], None):
+            call_definition_entity_type = calldata["entity_type"]
+
+            try:
+                entity_type = params[call_definition_entity_type]
+            except KeyError:
+                entity_type = call_definition_entity_type
+
+            if isinstance(params[calldata["command_primary"]], list):
+                res = []
+                for entity_id in params[calldata["command_primary"]]:
+                    res.append(
+                        self.transform_response_from_entity(
+                            entity_type, entity_id
+                        )
+                    )
+            else:
+                entity_id = params[calldata["command_primary"]]
+                res = self.transform_response_from_entity(
+                    entity_type, entity_id
+                )
+            return {"data": res}
+        else:
+            return {"data": None}
+
+    def transform_response_from_command_or_query(
+        self, response, calldata: ViewMapping, params
+    ) -> dict:
+        """
+        Transforms a response from a command to a proper JSONApi structure
 
         Args:
             response (Optional[Entity]): The response from our query or command
@@ -176,32 +274,8 @@ class JSONAPIEntityView:
 
         """
         if calldata["cq"] == "command":
-            try:
-                if (
-                    calldata["command_primary"]
-                    and params[calldata["command_primary"]]
-                ):
-                    link = self.create_link_from_entity(
-                        entity_type=calldata["entity_type"],
-                        entity_id=params[calldata["command_primary"]],
-                    )
-
-                    if link:
-                        links_block = {"links": {"self": link}}
-                    else:
-                        links_block = {}
-
-                    return {
-                        "data": {
-                            **links_block,
-                            "type": calldata["entity_type"],
-                            "id": params[calldata["command_primary"]],
-                        }
-                    }
-            except KeyError:
-                return {"data": None}
-
-        if calldata["cq"] == "query":
+            return self.transform_response_from_command(calldata, params)
+        elif calldata["cq"] == "query":
             if response is None:
                 return {"data": None}
             else:
@@ -210,12 +284,14 @@ class JSONAPIEntityView:
                     "links": {"self": self.request.current_route_path()},
                 }
 
+        assert_never(calldata["cq"])  # pragma: no cover
+
     def _collect_params_as_array(self, k, v, params):
         for pk, pv in self.request.params.items():
-            match = re.search(k + r"\[(.+)\]", pk)
+            match = re.search(r"^" + k + r"\[([^\]]+)\]$", pk)
 
             if match:
-                if k not in params:
+                if v not in params:
                     params[v] = {}
 
                 params[v][match[1]] = pv
@@ -296,28 +372,45 @@ class JSONAPIEntityView:
 
     def create_link_from_entity(
         self, entity=None, entity_type=None, entity_id=None
-    ):
+    ) -> Optional[str]:
         log.debug(
             "create_link_from_entity called, but not overridden in view class"
         )
+        return None
 
     def _extract_paging_params(self, calldata, request_params):
         final_params = {}
-        final_params["page"] = request_params["page"]
-        final_params["page_size"] = request_params["page_size"]
+        final_params["page"] = request_params.get("page", "1")
+        final_params["page_size"] = request_params.get(
+            "page_size", DEFAULT_PAGE_SIZE
+        )
 
-        self._check_paging_limit(
-            final_params["page_size"], calldata["paging"]["max_results"]
+        self._check_paging_limits(
+            int(final_params["page"]),
+            int(final_params["page_size"]),
+            int(calldata["paging"]["max_results"]),
         )
         return final_params
 
-    def _check_paging_limit(self, page_size, max_results):
-        if int(page_size) > int(max_results):
+    def _check_paging_limits(
+        self, page: int, page_size: int, max_results: int
+    ):
+        if page < 1:
+            raise HTTPBadRequest(
+                json={"errors": [{"title": "'page' must be >= 1"}]}
+            )
+
+        if page_size < 1:
+            raise HTTPBadRequest(
+                json={"errors": [{"title": "'page_size' must be >= 1"}]}
+            )
+
+        if page_size > max_results:
             raise HTTPBadRequest(
                 json={
                     "errors": [
                         {
-                            "title": f"Page size {page_size} is bigger than max allowed size {max_results}."
+                            "title": f"Requested page size {page_size} > max size {max_results}."
                         }
                     ]
                 }
